@@ -1,15 +1,24 @@
 """Test pysma sensors."""
 # from homeassistant.loader import async_get_dhcp
 import logging
-import aiohttp
 from json import loads
 from unittest.mock import patch
 
+import aiohttp
 import pytest
 
-import pysma
+from pysma import SMA
+from pysma.const import (
+    DEVCLASS_INVERTER,
+    ENERGY_METER_VIA_INVERTER,
+    JMESPATH_VAL,
+    JMESPATH_VAL_IDX,
+    OPTIMIZERS_VIA_INVERTER,
+)
+from pysma.definitions import sensor_map
+from pysma.sensor import Sensor, Sensors
 
-from . import MOCK_L10N, MOCK_DEVICE, mock_aioresponse  # noqa: F401
+from . import MOCK_DEVICE, MOCK_L10N, mock_aioresponse  # noqa: F401
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -42,8 +51,8 @@ SB_2_5 = loads(
 def sensors():
     """Fixture to create some sensors."""
     yield [
-        (402, True, pysma.Sensor("6400_00262200", "s_402", "W")),
-        (3514, True, pysma.Sensor("6400_00260100", "s_3514", "W", 1000)),
+        (402, True, Sensor("6400_00262200", "s_402", "W")),
+        (3514, True, Sensor("6400_00260100", "s_3514", "W", 1000)),
     ]
 
 
@@ -56,7 +65,7 @@ class Test_sensor_class:
             assert sens.path is None
             assert sens.extract_value(SB_1_5) is change
             assert sens.value == val
-            assert sens.path == pysma.JMESPATH_VAL_IDX.format(sens.key_idx)
+            assert sens.path == JMESPATH_VAL_IDX.format("1", sens.key_idx)
 
             assert sens.extract_value(SB_1_5) is False
 
@@ -66,13 +75,13 @@ class Test_sensor_class:
             assert sens.path is None
             assert sens.extract_value(SB_2_5) is change
             assert sens.value == val
-            assert sens.path == pysma.JMESPATH_VAL
+            assert sens.path == JMESPATH_VAL
 
             assert sens.extract_value(SB_2_5) is False
 
     def test_null(self):
         """Test a null or None result."""
-        sens = pysma.Sensor("6100_40263F00", "s_null", "W")
+        sens = Sensor("6100_40263F00", "s_null", "W")
         assert (
             sens.extract_value({"result": {"_": {"6100_40263F00": {"val": None}}}})
             is False
@@ -88,41 +97,41 @@ class Test_sensor_class:
         assert sens.extract_value({"result": {"_": {}}}) is False
         assert sens.value is None
 
-    @patch("pysma._LOGGER.warning")
+    @patch("pysma.sensor._LOGGER.warning")
     def test_default_no_duplicates(self, mock_warn):
         """Ensure warning on duplicates."""
-        sen = pysma.Sensors()
-        assert len(sen) > 25
+        sen = Sensors(sensor_map[DEVCLASS_INVERTER])
+        assert len(sen) > 20
         assert len(sen) < 50
         assert mock_warn.call_count == 0
         # Add duplicate frequency
-        news = pysma.Sensor("key1", "frequency", "")
+        news = Sensor("key1", "frequency", "")
         sen.add(news)
         assert mock_warn.call_count == 1
         assert sen[news.name] == news
         # Add duplicate freq, key should not be raised
-        sen.add(pysma.Sensor("6100_00465700", "frequency", ""))
+        sen.add(Sensor("6100_00465700", "frequency", ""))
         assert mock_warn.call_count == 2
         # Add duplicate freq key only
-        sen.add(pysma.Sensor("6100_00465700", "f001", ""))
+        sen.add(Sensor("6100_00465700", "f001", ""))
         assert mock_warn.call_count == 3
         # Test different key_idx only
-        sen.add(pysma.Sensor("key1_0", "frequency_0", ""))
+        sen.add(Sensor("key1_0", "frequency_0", ""))
         assert mock_warn.call_count == 3
-        sen.add(pysma.Sensor("key1_1", "frequency_1", ""))
+        sen.add(Sensor("key1_1", "frequency_1", ""))
         assert mock_warn.call_count == 3
 
-    @patch("pysma._LOGGER.warning")
+    @patch("pysma.sensor._LOGGER.warning")
     def test_type_error(self, mock_warn):
         """Ensure TypeError on not isinstance."""
-        sen = pysma.Sensors()
+        sen = Sensors()
         with pytest.raises(TypeError):
             sen.add("This is not a Sensor")
 
-    @patch("pysma._LOGGER.warning")
+    @patch("pysma.sensor._LOGGER.warning")
     def test_default_jmes(self, mock_warn):
         """Ensure default sensors are ok."""
-        sens = pysma.Sensors()
+        sens = Sensors(sensor_map[DEVCLASS_INVERTER])
         for sen in sens:
             sen.extract_value(SB_1_5)
         assert mock_warn.called
@@ -134,10 +143,11 @@ class Test_SMA_class:
         self.host = "1.1.1.1"
         self.base_url = f"http://{self.host}"
         mock_aioresponse.get(
-            f"{self.base_url}/data/l10n/en-US.json",
-            payload=MOCK_L10N,
+            f"{self.base_url}/data/l10n/en-US.json", payload=MOCK_L10N, repeat=True
+        )
+        mock_aioresponse.post(
+            f"{self.base_url}/dyn/logout.json?sid=ABCD", payload={}, repeat=True
         )
-        mock_aioresponse.post(f"{self.base_url}/dyn/logout.json?sid=ABCD", payload={})
 
     @patch("pysma._LOGGER.warning")
     async def test_session(self, mock_warn, mock_aioresponse):  # noqa: F811
@@ -145,7 +155,7 @@ class Test_SMA_class:
             f"{self.base_url}/dyn/login.json", payload={"result": {"sid": "ABCD"}}
         )
         session = aiohttp.ClientSession()
-        sma = pysma.SMA(session, self.host, "extralongpassword")
+        sma = SMA(session, self.host, "extralongpassword")
         assert await sma.new_session()
 
         assert mock_warn.call_count == 1
@@ -157,13 +167,13 @@ class Test_SMA_class:
     async def test_new_session_invalid_group(self, mock_aioresponse):  # noqa: F811
         session = aiohttp.ClientSession()
         with pytest.raises(KeyError):
-            pysma.SMA(session, self.host, "pass", "invalid-group")
+            SMA(session, self.host, "pass", "invalid-group")
 
     async def test_new_session_fail(self, mock_aioresponse):  # noqa: F811
         mock_aioresponse.post(f"{self.base_url}/dyn/login.json", payload={"result": {}})
 
         session = aiohttp.ClientSession()
-        sma = pysma.SMA(session, self.host, "pass")
+        sma = SMA(session, self.host, "pass")
         assert not await sma.new_session()
 
     async def test_device_info(self, mock_aioresponse):  # noqa: F811
@@ -197,7 +207,7 @@ class Test_SMA_class:
             },
         )
         session = aiohttp.ClientSession()
-        sma = pysma.SMA(session, self.host, "pass")
+        sma = SMA(session, self.host, "pass")
         result = await sma.device_info()
         assert result
         assert result == MOCK_DEVICE
@@ -220,7 +230,7 @@ class Test_SMA_class:
             },
         )
         session = aiohttp.ClientSession()
-        sma = pysma.SMA(session, self.host, "pass")
+        sma = SMA(session, self.host, "pass")
         assert await sma.new_session()
         result = await sma.device_info()
         assert result
@@ -238,7 +248,165 @@ class Test_SMA_class:
             payload={},
         )
         session = aiohttp.ClientSession()
-        sma = pysma.SMA(session, self.host, "pass")
+        sma = SMA(session, self.host, "pass")
         assert await sma.new_session()
         result = await sma.device_info()
         assert not result
+
+    async def test_get_devclass(self, mock_aioresponse):  # noqa: F811
+        mock_aioresponse.post(
+            f"{self.base_url}/dyn/login.json",
+            payload={"result": {"sid": "ABCD"}},
+        )
+        mock_aioresponse.post(
+            f"{self.base_url}/dyn/getValues.json?sid=ABCD",
+            payload={"result": {"0199-xxxxx385": {}}},
+        )
+
+        session = aiohttp.ClientSession()
+        sma = SMA(session, self.host, "pass")
+        assert await sma.get_devclass() is None
+
+        mock_aioresponse.post(
+            f"{self.base_url}/dyn/getValues.json?sid=ABCD",
+            payload={
+                "result": {
+                    "0199-xxxxx385": {
+                        "6180_08214800": {
+                            "1": [{"val": [{"tag": "123"}]}],
+                        },
+                    }
+                }
+            },
+        )
+
+        assert await sma.get_devclass() == DEVCLASS_INVERTER
+
+        sma.devclass = None
+        mock_aioresponse.post(
+            f"{self.base_url}/dyn/getValues.json?sid=ABCD",
+            payload={
+                "result": {
+                    "0199-xxxxx385": {
+                        "6180_08214800": {},
+                    }
+                }
+            },
+        )
+        assert await sma.get_devclass() is None
+
+        sma.devclass = None
+        mock_aioresponse.post(
+            f"{self.base_url}/dyn/getValues.json?sid=ABCD",
+            payload={
+                "result": {
+                    "0199-xxxxx385": {
+                        "6180_08214800": {
+                            "val": "402",
+                        },
+                    }
+                }
+            },
+        )
+        assert await sma.get_devclass() is None
+
+        mock_aioresponse.post(
+            f"{self.base_url}/dyn/getValues.json?sid=ABCD",
+            payload={
+                "result": {
+                    "0199-xxxxx385": {
+                        "6800_12345678": {
+                            "1": [{"val": "value1"}],
+                            "7": [{"val": "value2"}],
+                        },
+                    }
+                }
+            },
+        )
+
+        with pytest.raises(KeyError):
+            await sma.get_devclass()
+
+        mock_aioresponse.post(
+            f"{self.base_url}/dyn/getValues.json?sid=ABCD",
+            payload={
+                "result": {
+                    "0199-xxxxx385": {
+                        "6180_08214800": {
+                            "1": [{"val": [{"tag": "123"}]}],
+                        },
+                    }
+                }
+            },
+        )
+
+        sma.devclass = None
+        assert await sma.get_devclass("BOGUS_BODY") == DEVCLASS_INVERTER
+
+        mock_aioresponse.post(
+            f"{self.base_url}/dyn/getValues.json?sid=ABCD",
+            payload={
+                "result": {
+                    "0199-xxxxx385": {
+                        "6180_08214800": {
+                            "1": [{"val": [{"tag": "123"}]}],
+                        },
+                    }
+                }
+            },
+        )
+
+        assert await sma.get_devclass({}) == DEVCLASS_INVERTER
+
+        sma.devclass = "test"
+        assert await sma.get_devclass() == "test"
+
+    async def test_get_sensors(self, mock_aioresponse):  # noqa: F811
+        mock_aioresponse.post(
+            f"{self.base_url}/dyn/login.json", payload={"result": {"sid": "ABCD"}}
+        )
+        mock_aioresponse.post(
+            f"{self.base_url}/dyn/getValues.json?sid=ABCD",
+            payload={
+                "result": {
+                    "0199-xxxxx385": {
+                        "6180_08214800": {
+                            "1": [{"val": [{"tag": "123"}]}],
+                        },
+                        "6800_008AA300": {
+                            "1": [{"val": "0123456"}],
+                        },
+                        "6800_10852600": {
+                            "1": [
+                                {"val": "1"},
+                                {"val": "2"},
+                                {"val": None},
+                            ]
+                        },
+                    }
+                }
+            },
+            repeat=True,
+        )
+
+        session = aiohttp.ClientSession()
+        sma = SMA(session, self.host, "pass")
+        assert len(await sma.get_sensors()) == (
+            len(sensor_map[DEVCLASS_INVERTER])
+            + len(sensor_map[ENERGY_METER_VIA_INVERTER])
+            + (len(sensor_map[OPTIMIZERS_VIA_INVERTER]) * 2)
+        )
+
+    async def test_get_sensors_no_result_body(self, mock_aioresponse):  # noqa: F811
+        mock_aioresponse.post(
+            f"{self.base_url}/dyn/login.json", payload={"result": {"sid": "ABCD"}}
+        )
+        mock_aioresponse.post(
+            f"{self.base_url}/dyn/getValues.json?sid=ABCD",
+            payload={},
+            repeat=True,
+        )
+
+        session = aiohttp.ClientSession()
+        sma = SMA(session, self.host, "pass")
+        assert len(await sma.get_sensors()) == len(sensor_map[DEVCLASS_INVERTER])
